var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
function resolveLocale(unit, locales) {
    var localeData = UnifiedNumberFormat.__unitLocaleData__;
    var resolvedLocales = (Array.isArray(locales)
        ? locales
        : [locales]).filter(function (s) { return typeof s === 'string'; });
    var i, len, localeParts, data;
    var supportedLocales = [];
    // Using the set of locales + the default locale, we look for the first one
    // which that has been registered. When data does not exist for a locale, we
    // traverse its ancestors to find something that's been registered within
    // its hierarchy of locales. Since we lack the proper `parentLocale` data
    // here, we must take a naive approach to traversal.
    for (i = 0, len = resolvedLocales.length; i < len; i += 1) {
        localeParts = resolvedLocales[i].toLowerCase().split('-');
        while (localeParts.length) {
            if (localeData[unit]) {
                data = localeData[unit][localeParts.join('-')];
                if (data) {
                    // Return the normalized locale string; e.g., we return "en-US",
                    // instead of "en-us".
                    supportedLocales.push(data.locale);
                    break;
                }
                localeParts.pop();
            }
        }
    }
    return supportedLocales;
}
var UnifiedNumberFormat = /** @class */ (function () {
    function UnifiedNumberFormat(locale, _a) {
        if (_a === void 0) { _a = {}; }
        var style = _a.style, unit = _a.unit, unitDisplay = _a.unitDisplay, options = __rest(_a, ["style", "unit", "unitDisplay"]);
        this.unit = undefined;
        this.unitDisplay = undefined;
        if (style === 'unit') {
            if (!unit) {
                throw new TypeError('Unit is required for `style: unit`');
            }
            this.unit = unit;
            this.unitDisplay = unitDisplay || 'short';
            this.locale = resolveLocale(unit, [locale])[0];
        }
        this.nf = new Intl.NumberFormat(locale, __assign({}, options, { style: style === 'unit' ? 'decimal' : style }));
    }
    UnifiedNumberFormat.prototype.format = function (num) {
        var formattedNum = this.nf.format(num);
        if (this.unit) {
            var locale = this.nf.resolvedOptions().locale;
            var patternData = UnifiedNumberFormat.__unitLocaleData__[this.unit][locale];
            var pl = new Intl.PluralRules(locale).select(num);
            return patternData.fields[this.unitDisplay][pl === 'one' ? 'one' : 'other'].replace('{0}', formattedNum);
        }
        return formattedNum;
    };
    UnifiedNumberFormat.prototype.formatToParts = function (num) {
        return this.nf.formatToParts(num);
    };
    UnifiedNumberFormat.supportedLocalesOf = function () {
        var _a;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return (_a = Intl.NumberFormat).supportedLocalesOf.apply(_a, args);
    };
    UnifiedNumberFormat.__addUnitLocaleData = function (unit) {
        var data = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            data[_i - 1] = arguments[_i];
        }
        if (!UnifiedNumberFormat.__unitLocaleData__[unit]) {
            UnifiedNumberFormat.__unitLocaleData__[unit] = {};
        }
        for (var _a = 0, data_1 = data; _a < data_1.length; _a++) {
            var datum = data_1[_a];
            if (!(datum && datum.locale)) {
                throw new Error('Locale data provided to UnifiedNumberFormat is missing a ' +
                    '`locale` property value');
            }
            UnifiedNumberFormat.__unitLocaleData__[unit][datum.locale.toLowerCase()] = datum;
        }
    };
    UnifiedNumberFormat.__unitLocaleData__ = {};
    return UnifiedNumberFormat;
}());
export default UnifiedNumberFormat;
